#*******************************************************************************
# Copyright (C) 2007,2008 Novell, Inc. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of Novell, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
#*****************************************************************************/
"""Python Provider for OMC_ComputerSystem

Instruments the CIM class OMC_ComputerSystem

"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2
from socket import getfqdn

class OMC_ComputerSystemProvider(CIMProvider2):
    """Instrument the CIM class OMC_ComputerSystem 

    OMC  Computer System
    
    """

    def get_instance(self, env, model):
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        
        #model['RequestedState'] = # TODO (type = pywbem.Uint16 self.Values.RequestedState) (default=12L)
        #model['HealthState'] = # TODO (type = pywbem.Uint16 self.Values.HealthState) 
        #model['PowerState'] = # TODO (type = pywbem.Uint16 self.Values.PowerState) 
        #model['StatusDescriptions'] = # TODO (type = [unicode,]) 
        #model['Dedicated'] = # TODO (type = [pywbem.Uint16,] self.Values.Dedicated) 
        #model['OtherEnabledState'] = # TODO (type = unicode) 
        #model['PrimaryOwnerName'] = # TODO (type = unicode) 
        #model['OtherIdentifyingInfo'] = # TODO (type = [unicode,]) 
        #model['PowerManagementSupported'] = # TODO (type = bool) 
        #model['IdentifyingDescriptions'] = # TODO (type = [unicode,]) 
        #model['LastLoadInfo'] = # TODO (type = unicode) 
        #model['WakeUpType'] = # TODO (type = pywbem.Uint16 self.Values.WakeUpType) 
        #model['OperationalStatus'] = # TODO (type = [pywbem.Uint16,] self.Values.OperationalStatus) 
        #model['PrimaryOwnerContact'] = # TODO (type = unicode) 
        #model['OtherDedicatedDescriptions'] = # TODO (type = [unicode,]) 
        #model['Status'] = # TODO (type = unicode self.Values.Status) 
        #model['Description'] = # TODO (type = unicode) 
        #model['InstallDate'] = # TODO (type = pywbem.CIMDateTime) 
        #model['EnabledDefault'] = # TODO (type = pywbem.Uint16 self.Values.EnabledDefault) (default=2L)
        #model['EnabledState'] = # TODO (type = pywbem.Uint16 self.Values.EnabledState) (default=5L)
        #model['TimeOfLastStateChange'] = # TODO (type = pywbem.CIMDateTime) 
        #model['InitialLoadInfo'] = # TODO (type = [unicode,]) 
        #model['ElementName'] = # TODO (type = unicode) 
        #model['Roles'] = # TODO (type = [unicode,]) 
        #model['ResetCapability'] = # TODO (type = pywbem.Uint16 self.Values.ResetCapability) 
        model['NameFormat'] = self.Values.NameFormat.IP
        #model['Caption'] = # TODO (type = unicode) 
        #model['PowerManagementCapabilities'] = # TODO (type = [pywbem.Uint16,] self.Values.PowerManagementCapabilities) 
        return model

    def enum_instances(self, env, model, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        model.path.update({'CreationClassName':None, 'Name':None})

        # Key properties    
        model['CreationClassName'] = 'OMC_ComputerSystem'    
        model['Name'] = getfqdn()
        if keys_only:
            yield model
        else:
            try:
                yield self.get_instance(env, model)
            except pywbem.CIMError, (num, msg):
                if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                               pywbem.CIM_ERR_ACCESS_DENIED):
                    raise

    def set_instance(self, env, instance, modify_existing):

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        return instance

    def delete_instance(self, env, instance_name):

        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        # TODO delete the resource
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
    def cim_method_requeststatechange(self, env, object_name,
                                      param_requestedstate,
                                      param_timeoutperiod):
        """Implements OMC_ComputerSystem.RequestStateChange()

        Requests that the state of the element be changed to the value
        specified in the RequestedState parameter. When the requested
        state change takes place, the EnabledState and RequestedState of
        the element will be the same. Invoking the RequestStateChange
        method multiple times could result in earlier requests being
        overwritten or lost.  If 0 is returned, then the task completed
        successfully and the use of ConcreteJob was not required. If 4096
        (0x1000) is returned, then the task will take some time to
        complete, ConcreteJob will be created, and its reference returned
        in the output parameter Job. Any other return code indicates an
        error condition.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method RequestStateChange() 
            should be invoked.
        method -- A pywbem.CIMMethod representing the method meta-data
        param_requestedstate --  The input parameter RequestedState (type pywbem.Uint16 self.Values.RequestStateChange.RequestedState) 
            The state requested for the element. This information will be
            placed into the RequestedState property of the instance if the
            return code of the RequestStateChange method is 0 ('Completed
            with No Error'), 3 ('Timeout'), or 4096 (0x1000) ('Job
            Started'). Refer to the description of the EnabledState and
            RequestedState properties for the detailed explanations of the
            RequestedState values.
            
        param_timeoutperiod --  The input parameter TimeoutPeriod (type pywbem.CIMDateTime) 
            A timeout period that specifies the maximum amount of time that
            the client expects the transition to the new state to take.
            The interval format must be used to specify the TimeoutPeriod.
            A value of 0 or a null parameter indicates that the client has
            no time requirements for the transition.  If this property
            does not contain 0 or null and the implementation does not
            support this parameter, a return code of 'Use Of Timeout
            Parameter Not Supported' must be returned.
            

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.RequestStateChange)
        and a dictionary with the out-parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...)) 
            Reference to the job (can be null if the task is completed).
            

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_requeststatechange()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = {}
        #out_params['job'] = # TODO (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))
        rval = None # TODO (type pywbem.Uint32 self.Values.RequestStateChange)
        return (rval, out_params)
        
    def cim_method_setpowerstate(self, env, object_name, 
                                 param_powerstate,
                                 param_time):
        """Implements OMC_ComputerSystem.SetPowerState()

        Sets the power state of the computer. The use of this method has
        been deprecated. Instead, use the SetPowerState method in the
        associated PowerManagementService class.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method SetPowerState() 
            should be invoked.
        method -- A pywbem.CIMMethod representing the method meta-data
        param_powerstate --  The input parameter PowerState (type pywbem.Uint32 self.Values.SetPowerState.PowerState) 
            The Desired state for the COmputerSystem.
            
        param_time --  The input parameter Time (type pywbem.CIMDateTime) 
            Time indicates when the power state should be set, either as a
            regular date-time value or as an interval value (where the
            interval begins when the method invocation is received.
            

        Returns a two-tuple containing the return value (type pywbem.Uint32)
        and a dictionary with the out-parameters

        Output parameters: none

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_setpowerstate()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = {}
        rval = None # TODO (type pywbem.Uint32)
        return (rval, out_params)
        
    class Values(object):
        class Status(object):
            OK = 'OK'
            Error = 'Error'
            Degraded = 'Degraded'
            Unknown = 'Unknown'
            Pred_Fail = 'Pred Fail'
            Starting = 'Starting'
            Stopping = 'Stopping'
            Service = 'Service'
            Stressed = 'Stressed'
            NonRecover = 'NonRecover'
            No_Contact = 'No Contact'
            Lost_Comm = 'Lost Comm'
            Stopped = 'Stopped'

        class RequestedState(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            No_Change = pywbem.Uint16(5)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)
            Not_Applicable = pywbem.Uint16(12)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class HealthState(object):
            Unknown = pywbem.Uint16(0)
            OK = pywbem.Uint16(5)
            Degraded_Warning = pywbem.Uint16(10)
            Minor_failure = pywbem.Uint16(15)
            Major_failure = pywbem.Uint16(20)
            Critical_failure = pywbem.Uint16(25)
            Non_recoverable_error = pywbem.Uint16(30)
            # DMTF_Reserved = ..

        class SetPowerState(object):
            class PowerState(object):
                Full_Power = pywbem.Uint32(1)
                Power_Save___Low_Power_Mode = pywbem.Uint32(2)
                Power_Save___Standby = pywbem.Uint32(3)
                Power_Save___Other = pywbem.Uint32(4)
                Power_Cycle = pywbem.Uint32(5)
                Power_Off = pywbem.Uint32(6)
                Hibernate = pywbem.Uint32(7)
                Soft_Off = pywbem.Uint32(8)

        class PowerState(object):
            Unknown = pywbem.Uint16(0)
            Full_Power = pywbem.Uint16(1)
            Power_Save___Low_Power_Mode = pywbem.Uint16(2)
            Power_Save___Standby = pywbem.Uint16(3)
            Power_Save___Unknown = pywbem.Uint16(4)
            Power_Cycle = pywbem.Uint16(5)
            Power_Off = pywbem.Uint16(6)
            Power_Save___Warning = pywbem.Uint16(7)
            Power_Save___Hibernate = pywbem.Uint16(8)
            Power_Save___Soft_Off = pywbem.Uint16(9)

        class EnabledDefault(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            No_Default = pywbem.Uint16(7)
            Quiesce = pywbem.Uint16(9)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class EnabledState(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shutting_Down = pywbem.Uint16(4)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            In_Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Starting = pywbem.Uint16(10)
            # DMTF_Reserved = 11..32767
            # Vendor_Reserved = 32768..65535

        class ResetCapability(object):
            Other = pywbem.Uint16(1)
            Unknown = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Enabled = pywbem.Uint16(4)
            Not_Implemented = pywbem.Uint16(5)

        class Dedicated(object):
            Not_Dedicated = pywbem.Uint16(0)
            Unknown = pywbem.Uint16(1)
            Other = pywbem.Uint16(2)
            Storage = pywbem.Uint16(3)
            Router = pywbem.Uint16(4)
            Switch = pywbem.Uint16(5)
            Layer_3_Switch = pywbem.Uint16(6)
            Central_Office_Switch = pywbem.Uint16(7)
            Hub = pywbem.Uint16(8)
            Access_Server = pywbem.Uint16(9)
            Firewall = pywbem.Uint16(10)
            Print = pywbem.Uint16(11)
            I_O = pywbem.Uint16(12)
            Web_Caching = pywbem.Uint16(13)
            Management = pywbem.Uint16(14)
            Block_Server = pywbem.Uint16(15)
            File_Server = pywbem.Uint16(16)
            Mobile_User_Device = pywbem.Uint16(17)
            Repeater = pywbem.Uint16(18)
            Bridge_Extender = pywbem.Uint16(19)
            Gateway = pywbem.Uint16(20)
            Storage_Virtualizer = pywbem.Uint16(21)
            Media_Library = pywbem.Uint16(22)
            ExtenderNode = pywbem.Uint16(23)
            NAS_Head = pywbem.Uint16(24)
            Self_contained_NAS = pywbem.Uint16(25)
            UPS = pywbem.Uint16(26)
            IP_Phone = pywbem.Uint16(27)
            Management_Controller = pywbem.Uint16(28)
            Chassis_Manager = pywbem.Uint16(29)
            # DMTF_Reserved = 30..32567
            # Vendor_Reserved = 32568..65535

        class NameFormat(object):
            Other = 'Other'
            IP = 'IP'
            Dial = 'Dial'
            HID = 'HID'
            NWA = 'NWA'
            HWA = 'HWA'
            X25 = 'X25'
            ISDN = 'ISDN'
            IPX = 'IPX'
            DCC = 'DCC'
            ICD = 'ICD'
            E_164 = 'E.164'
            SNA = 'SNA'
            OID_OSI = 'OID/OSI'
            WWN = 'WWN'
            NAA = 'NAA'

        class WakeUpType(object):
            Reserved = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Unknown = pywbem.Uint16(2)
            APM_Timer = pywbem.Uint16(3)
            Modem_Ring = pywbem.Uint16(4)
            LAN_Remote = pywbem.Uint16(5)
            Power_Switch = pywbem.Uint16(6)
            PCI_PME_ = pywbem.Uint16(7)
            A_C_Power_Restored = pywbem.Uint16(8)

        class OperationalStatus(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            OK = pywbem.Uint16(2)
            Degraded = pywbem.Uint16(3)
            Stressed = pywbem.Uint16(4)
            Predictive_Failure = pywbem.Uint16(5)
            Error = pywbem.Uint16(6)
            Non_Recoverable_Error = pywbem.Uint16(7)
            Starting = pywbem.Uint16(8)
            Stopping = pywbem.Uint16(9)
            Stopped = pywbem.Uint16(10)
            In_Service = pywbem.Uint16(11)
            No_Contact = pywbem.Uint16(12)
            Lost_Communication = pywbem.Uint16(13)
            Aborted = pywbem.Uint16(14)
            Dormant = pywbem.Uint16(15)
            Supporting_Entity_in_Error = pywbem.Uint16(16)
            Completed = pywbem.Uint16(17)
            Power_Mode = pywbem.Uint16(18)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class PowerManagementCapabilities(object):
            Unknown = pywbem.Uint16(0)
            Not_Supported = pywbem.Uint16(1)
            Disabled = pywbem.Uint16(2)
            Enabled = pywbem.Uint16(3)
            Power_Saving_Modes_Entered_Automatically = pywbem.Uint16(4)
            Power_State_Settable = pywbem.Uint16(5)
            Power_Cycling_Supported = pywbem.Uint16(6)
            Timed_Power_On_Supported = pywbem.Uint16(7)

        class RequestStateChange(object):
            Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown_or_Unspecified_Error = pywbem.Uint32(2)
            Cannot_complete_within_Timeout_Period = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Invalid_State_Transition = pywbem.Uint32(4097)
            Use_of_Timeout_Parameter_Not_Supported = pywbem.Uint32(4098)
            Busy = pywbem.Uint32(4099)
            # Method_Reserved = 4100..32767
            # Vendor_Specific = 32768..65535
            class RequestedState(object):
                Enabled = pywbem.Uint16(2)
                Disabled = pywbem.Uint16(3)
                Shut_Down = pywbem.Uint16(4)
                Offline = pywbem.Uint16(6)
                Test = pywbem.Uint16(7)
                Defer = pywbem.Uint16(8)
                Quiesce = pywbem.Uint16(9)
                Reboot = pywbem.Uint16(10)
                Reset = pywbem.Uint16(11)
                # DMTF_Reserved = ..
                # Vendor_Reserved = 32768..65535

## end of class OMC_ComputerSystemProvider



def get_providers(env): # register provider
    omc_computersystem_prov = OMC_ComputerSystemProvider() 
    return {'OMC_ComputerSystem': omc_computersystem_prov}




